# Copyright 2021 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

from odoo import api, fields, models


class AccCounter(models.Model):
    _inherit = "acc.counter"

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------
    acc_account_surplus_ids = fields.One2many(
        "acc.account",
        "acc_delivery_id",
        domain=[("is_account_buyer", "=", True)],
        string="Factures Surplus liées",
    )
    acc_account_delivery_ids = fields.One2many(
        "acc.account", "acc_delivery_id", string="Factures liées"
    )
    acc_account_injection_ids = fields.One2many(
        "acc.account",
        "acc_injection_id",
        domain=[("is_account_buyer", "=", False)],
        string="Factures liées",
    )
    acc_account_delivery_count = fields.Integer(
        compute="_compute_acc_account_count", string="Nombre de factures clients"
    )
    acc_account_injection_count = fields.Integer(
        compute="_compute_acc_account_count", string="Nombre de factures producteurs"
    )
    acc_account_surplus_count = fields.Integer(
        compute="_compute_acc_account_count",
        string="Nombre de factures surplus producteurs",
    )
    is_account_auto_activate = fields.Boolean(
        "Facturation de l’autoconsommation automatisée", default=True
    )
    is_account_surplus_activate = fields.Boolean(
        "Facturation du surplus automatisée", default=True
    )
    sale_price = fields.Float("Prix de vente par défaut", digits="Sale Price")
    price_accise = fields.Float("Tarif ACCISE par défaut", digits="Sale Price")

    # ------------------------------------------------------
    # SQL Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Default methods
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Computed fields / Search Fields
    # ------------------------------------------------------
    @api.depends("acc_account_delivery_ids", "acc_account_delivery_ids")
    def _compute_acc_account_count(self):
        """
        Fonction permettant de calculer:
         - le nombre de factures de vente par point d'injection,
         - le nombre de factures de vente par point de soutirage,
         - le nombre de factures Achat par point d'injection
        """
        for res in self:
            res.acc_account_delivery_count = self.env["acc.account"].search_count(
                [("acc_delivery_id", "=", res.id)]
            )
            res.acc_account_injection_count = self.env["acc.account"].search_count(
                [("acc_injection_id", "=", res.id), ("is_account_buyer", "=", False)]
            )
            res.acc_account_surplus_count = self.env["acc.account"].search_count(
                [("acc_injection_id", "=", res.id), ("is_account_buyer", "=", True)]
            )

    # ------------------------------------------------------
    # Onchange / Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # CRUD methods (ORM overrides)
    # ------------------------------------------------------
    @api.model_create_multi
    def create(self, vals_list):
        res = super(AccCounter, self).create(vals_list)
        """
        Si un point de livraison possède une opération, passage des valeurs
        du prix de de vente par défaut et du tarif ACCISE par défaut
        """
        if res.acc_operation_id and res.is_delivery:
            res.sale_price = res.acc_operation_id.sale_price
            res.price_accise = res.acc_operation_id.price_accise
        return res

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------
    def action_view_acc_injection_accounts(self):
        """
        Action permettant d'accéder auc factures de vente du point d'injection
        """
        self.ensure_one()
        action = self.env["ir.actions.actions"]._for_xml_id(
            "acc_account.action_acc_account"
        )
        action["domain"] = [("acc_injection_id", "=", self.id)]
        return action

    def action_view_acc_delivery_accounts(self):
        """
        Action permettant d'accéder auc factures de vente du point de soutirage
        """
        self.ensure_one()
        action = self.env["ir.actions.actions"]._for_xml_id(
            "acc_account.action_acc_account"
        )
        action["domain"] = [("acc_delivery_id", "=", self.id)]
        return action

    # ------------------------------------------------------
    # Business methods
    # ------------------------------------------------------
