# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl).
from odoo import api, fields, models


class AccPriceConfWizard(models.TransientModel):
    _name = "acc.price.conf.wizard"
    _description = "Gestion des prix wizard"

    def _default_operation_id(self):
        return self.env.context.get("active_id")

    def _default_type(self):
        return self.env.context.get("type")

    def _default_price_conf_ids(self):
        # If click button on operation view
        type = self.env.context.get("type")
        if type == "buyer":
            inj_ids = self.env["acc.counter"].search(
                [
                    ("is_injection", "=", True),
                    ("acc_operation_id", "=", self.env.context.get("active_id")),
                    ("buyer_id", "!=", False),
                ]
            )

            return [
                (
                    0,
                    0,
                    {
                        "acc_buyer_id": inj.buyer_id.id,
                        "acc_injection_id": inj.id,
                        "acc_operation_id": self.env.context.get("active_id"),
                        "type": type,
                    },
                )
                for inj in inj_ids
            ]
        elif type == "sale" or type == "accise":
            inj_ids = self.env["acc.counter"].search(
                [
                    ("is_injection", "=", True),
                    ("acc_operation_id", "=", self.env.context.get("active_id")),
                ]
            )
            recs = self.env["acc.counter"].search(
                [
                    ("is_delivery", "=", True),
                    ("acc_operation_id", "=", self.env.context.get("active_id")),
                ]
            )

            return [
                (
                    0,
                    0,
                    {
                        "acc_delivery_id": rec.id,
                        "acc_injection_id": inj.id,
                        "acc_operation_id": self.env.context.get("active_id"),
                        "type": type,
                        "price": rec.sale_price if type == "sale" else rec.price_accise,
                    },
                )
                for rec in recs
                for inj in inj_ids
            ]
        else:
            # Objects selected from list view
            active_ids = self._context.get("active_ids")
            if active_ids:
                return [
                    (
                        0,
                        0,
                        {
                            "acc_delivery_id": self.env["acc.price.conf"]
                            .browse(rec)
                            .acc_delivery_id.id,
                            "acc_injection_id": self.env["acc.price.conf"]
                            .browse(rec)
                            .acc_injection_id.id,
                            "acc_operation_id": self.env["acc.price.conf"]
                            .browse(rec)
                            .acc_operation_id.id,
                            "type": self.env["acc.price.conf"].browse(rec).type,
                        },
                    )
                    for rec in active_ids
                ]

    acc_price_conf_ids = fields.Many2many(
        "acc.price.conf", default=lambda self: self._default_price_conf_ids()
    )
    start_date = fields.Date("Début de période", default=fields.Date.context_today)
    acc_operation_id = fields.Many2one("acc.operation", default=_default_operation_id)
    type = fields.Selection(
        [("sale", "Vente"), ("buyer", "Achat"), ("accise", "ACCISE")],
        default=_default_type,
    )

    @api.onchange("start_date")
    def _onchange_start_date(self):
        if self.start_date:
            for price in self.acc_price_conf_ids:
                price.start_date = self.start_date
